"""
This module is the same as `canvaslms.grades.conjunctavg` except that any 
submissions with grades other than A--F and P/F are treated as P. For instance, 
numeric grades (like points). Also, all submissions must have a date. This 
makes this module useful for including mandatory, ungraded surveys.
"""

import datetime as dt
from canvaslms.grades.conjunctavg import a2e_average
from canvaslms.cli import results
from canvasapi.exceptions import ResourceDoesNotExist

def summarize(user, assignments_list):
  """
  Extracts user's submissions for assignments in assingments_list to summarize 
  results into one grade and a grade date. Summarize by conjunctive average.

  If some submission lacks date, return ("F", None).
  """

  pf_grades = []
  a2e_grades = []
  dates = []
  graders = []

  for assignment in assignments_list:
    try:
      submission = assignment.get_submission(user,
                                            include=["submission_history"])
    except ResourceDoesNotExist:
      pf_grades.append("F")
      continue

    grade = submission.grade
    graders += results.all_graders(submission)

    if grade is None:
      grade = "F"

    if grade in "ABCDE":
      a2e_grades.append(grade)
    elif grade in "PF":
      pf_grades.append(grade)
    elif grade == "Fx":
      pf_grades.append("F")
    else:
      pf_grades.append("P")

    grade_date = submission.submitted_at or submission.graded_at

    if grade_date:
      grade_date = dt.date.fromisoformat(grade_date.split("T")[0])
      dates.append(grade_date)

  if all(map(lambda x: x == "P", pf_grades)):
    final_grade = "P"
    if a2e_grades:
      final_grade = a2e_average(a2e_grades)

  if dates:
    final_date = max(dates)
  else:
    final_date = None
    final_grade = None

  if len(dates) < len(pf_grades) + len(a2e_grades):
    final_grade = "F"

  return (final_grade, final_date, graders)

def summarize_group(assignments_list, users_list):
  """Summarizes a particular set of assignments (assignments_list) for all
  users in users_list"""

  for user in users_list:
    grade, grade_date, graders = summarize(user, assignments_list)
    yield [user, grade, grade_date, *graders]



"""
This package contains modules to summarize assignment groups in different ways. 
These modules are used with the `-S` option of the `results` command.

For a module to be used with the `canvaslms results -S module` option, the 
module must fulfil the following:

  1) It must contain a function named `summarize_group`.
  2) `summarize_group` must take two arguments:

       I) `assignment_list`, a list of `canvasapi.assignment.Assignment`
          objects. These assignments all belong to the same group, \ie their
          grades should be used to compute the student's final grade.

      II) `users_list`, a list of `canvasapi.user.User` objects. This is a
          list of users, i.e. students, for whom to compute the grades.

  3) The return value should be a list of lists. Each list should have the
     form `[user, grade, grade date, grader 1, ..., grader N]`.

For more details, see Chapter 11 of the `canvaslms.pdf` file found among the 
release files at:

  https://github.com/dbosk/canvaslms/releases
"""



from canvaslms.cli import submissions, users
import webbrowser

def grade_command(config, canvas, args):
  submission_list = submissions.process_submission_options(canvas, args)
  results = {}
  if args.grade:
    results["submission"] = {"posted_grade": args.grade}
  if args.message:
    results["comment"] = {"text_comment": args.message}
  if not args.grade and not args.message:
    for submission in submission_list:
      webbrowser.open(submissions.speedgrader(submission))
  else:
    for submission in submission_list:
      if args.verbose:
        id = f"{submission.assignment.course.course_code} " \
             f"{submission.assignment.name} {submission.user}"

        event = ""
        try:
          event += f" grade = {args.grade}"
        except:
          pass
        try:
          event += f" msg = '{args.message}'"
        except:
          pass

        print(f"{id}:{event}")

      submission.edit(**results)

def add_command(subp):
  """Adds grade command to the argparse subparser subp"""
  grade_parser = subp.add_parser("grade",
    help="Grades assignments (hic sunt dracones!)",
    description="Grades assignments. ***Hic sunt dracones [here be dragons]***: "
      "the regex matching is very powerful, "
      "be certain that you match what you think!")
  grade_parser.set_defaults(func=grade_command)
  submissions.add_submission_options(grade_parser, required=True)
  grade_options = grade_parser.add_argument_group(
    "arguments to set the grade and/or comment, "
    "if none given, opens SpeedGrader")
  grade_options.add_argument("-g", "--grade",
    help="The grade to set for the submissions")
  grade_options.add_argument("-m", "--message",
    help="A comment to the student")
  grade_parser.add_argument("-v", "--verbose",
    action="store_true", default=False,
    help="Increases verbosity, prints what grade is set "
         "for which assignment for which student.")
